@@ -52,7 +52,7 @@ static int test_conc66(MYSQL *my)
   rc= mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, "./my-conc66-test.cnf");
   check_mysql_rc(rc, mysql);
 
-  sprintf(query, "GRANT ALL ON %s.* TO 'conc66'@'%s' IDENTIFIED BY 'test\";#test'", schema, hostname ? hostname : "localhost");
+  sprintf(query, "GRANT ALL ON %s.* TO 'conc66'@'%s' IDENTIFIED BY 'test\";#test'", schema, this_host ? this_host : "localhost");
   rc= mysql_query(my, query);
   check_mysql_rc(rc, my);
   rc= mysql_query(my, "FLUSH PRIVILEGES");
@@ -60,11 +60,13 @@ static int test_conc66(MYSQL *my)
   if (!my_test_connect(mysql, hostname, NULL,
                              NULL, schema, port, socketname, 0))
   {
+    diag("user: %s", mysql->options.user);
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
   }
+    diag("user: %s", mysql->options.user);
   
-  sprintf(query, "DROP user conc66@%s", hostname ? hostname : "localhost");
+  sprintf(query, "DROP user 'conc66'@'%s'", this_host ? this_host : "localhost");
   rc= mysql_query(my, query);
 
   check_mysql_rc(rc, my);
@@ -83,6 +85,9 @@ static int test_bug20023(MYSQL *mysql)
   int sql_big_selects_5;
   int rc;
 
+  if (!is_mariadb)
+    return SKIP;
+
   if (mysql_get_server_version(mysql) < 50100) {
     diag("Test requires MySQL Server version 5.1 or above");
     return SKIP;
@@ -579,7 +584,6 @@ static int test_reconnect(MYSQL *mysql)
 
   diag("Thread_id before kill: %lu", mysql_thread_id(mysql1));
   mysql_kill(mysql, mysql_thread_id(mysql1));
-  sleep(4);
 
   mysql_ping(mysql1);
 
@@ -657,7 +661,7 @@ int test_connection_timeout(MYSQL *unused __attribute__((unused)))
   elapsed= time(NULL) - start;
   diag("elapsed: %lu", (unsigned long)elapsed);
   mysql_close(mysql);
-  FAIL_IF(elapsed > 2 * timeout, "timeout ignored")
+  FAIL_IF((unsigned int)elapsed > 2 * timeout, "timeout ignored")
   return OK;
 }
 
@@ -677,7 +681,7 @@ int test_connection_timeout2(MYSQL *unused __attribute__((unused)))
   elapsed= time(NULL) - start;
   diag("elapsed: %lu", (unsigned long)elapsed);
   mysql_close(mysql);
-  FAIL_IF(elapsed > 2 * timeout, "timeout ignored")
+  FAIL_IF((unsigned int)elapsed > 2 * timeout, "timeout ignored")
   return OK;
 }
 
@@ -702,7 +706,7 @@ int test_connection_timeout3(MYSQL *unused __attribute__((unused)))
   }
   elapsed= time(NULL) - start;
   diag("elapsed: %lu", (unsigned long)elapsed);
-  FAIL_IF(elapsed > timeout + 1, "timeout ignored")
+  FAIL_IF((unsigned int)elapsed > timeout + 1, "timeout ignored")
 
   mysql_close(mysql);
   mysql= mysql_init(NULL);
@@ -737,7 +741,6 @@ static int test_conc118(MYSQL *mysql)
   mysql->options.unused_1= 1;
 
   rc= mysql_kill(mysql, mysql_thread_id(mysql));
-  sleep(2);
 
   mysql_ping(mysql);
 
@@ -747,7 +750,6 @@ static int test_conc118(MYSQL *mysql)
   FAIL_IF(mysql->options.unused_1 != 1, "options got lost");
 
   rc= mysql_kill(mysql, mysql_thread_id(mysql));
-  sleep(2);
 
   mysql_ping(mysql);
   rc= mysql_query(mysql, "SET @a:=1");
@@ -834,7 +836,7 @@ static int test_get_options(MYSQL *unused __attribute__((unused)))
                       MYSQL_OPT_PROTOCOL, MYSQL_OPT_READ_TIMEOUT, MYSQL_OPT_WRITE_TIMEOUT, 0};
   my_bool options_bool[]= {MYSQL_OPT_RECONNECT, MYSQL_REPORT_DATA_TRUNCATION,
                            MYSQL_OPT_COMPRESS, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, MYSQL_SECURE_AUTH,
-#ifdef _WIN32
+#ifdef _WIN32    
     MYSQL_OPT_NAMED_PIPE,
 #endif
                           0};
@@ -877,7 +879,7 @@ static int test_get_options(MYSQL *unused __attribute__((unused)))
     mysql_options(mysql, options_char[i], char1);
     char2= NULL;
     mysql_get_optionv(mysql, options_char[i], (void *)&char2);
-    if (options_char[i] != MYSQL_SET_CHARSET_NAME)
+    if (options_char[i] != MYSQL_SET_CHARSET_NAME) 
       FAIL_IF(strcmp(char1, char2), "mysql_get_optionv (char) failed");
   }
 
@@ -981,8 +983,13 @@ static int test_sess_track_db(MYSQL *mysql)
   return OK;
 }
 
+
 static int test_unix_socket_close(MYSQL *unused __attribute__((unused)))
 {
+#ifdef _WIN32
+  diag("test does not run on Windows");
+  return SKIP;
+#else
   MYSQL *mysql= mysql_init(NULL);
   FILE *fp;
   int i;
@@ -1007,14 +1014,19 @@ static int test_unix_socket_close(MYSQL *unused __attribute__((unused)))
   }
   mysql_close(mysql);
   return OK;
+#endif
 }
 
+
 static int test_reset(MYSQL *mysql)
 {
   int rc;
   MYSQL_RES *res;
 
-  rc= mysql_query(mysql, "DROP TABLE IF exists t1");
+  if (mysql_get_server_version(mysql) < 100200)
+    return SKIP;
+
+  rc= mysql_query(mysql, "DROP TABLE IF EXISTS t1");
   check_mysql_rc(rc, mysql);
 
   rc= mysql_query(mysql, "CREATE TABLE t1 (a int)");
@@ -1026,14 +1038,9 @@ static int test_reset(MYSQL *mysql)
   FAIL_IF(mysql_affected_rows(mysql) != 3, "Expected 3 rows");
 
   rc= mysql_reset_connection(mysql);
-  if (rc && mysql_errno(mysql) == 1047)
-  {
-    diag("server doesn't support RESET_CONNECTION");
-    return SKIP;
-  }
   check_mysql_rc(rc, mysql);
 
-  FAIL_IF(mysql_affected_rows(mysql) != ~(unsigned long)0, "Expected 0 rows");
+  FAIL_IF(mysql_affected_rows(mysql) != ~(my_ulonglong)0, "Expected 0 rows");
 
   rc= mysql_query(mysql, "SELECT a FROM t1");
   check_mysql_rc(rc, mysql);
@@ -1072,6 +1079,14 @@ static int test_auth256(MYSQL *my)
   int rc;
   MYSQL_RES *res;
   my_ulonglong num_rows= 0;
+  char query[1024];
+
+  if (!mysql_client_find_plugin(mysql, "sha256_password", 3))
+  {
+    diag("sha256_password plugin not available");
+    mysql_close(mysql);
+    return SKIP;
+  }
 
   rc= mysql_query(my, "SELECT * FROM information_schema.plugins where plugin_name='sha256_password'");
   check_mysql_rc(rc, mysql);
@@ -1083,14 +1098,17 @@ static int test_auth256(MYSQL *my)
   if (!num_rows)
   {
     diag("server doesn't support sha256 authentication");
+    mysql_close(mysql);
     return SKIP;
   }
 
   rc= mysql_query(my, "DROP USER IF EXISTS sha256user@localhost");
   check_mysql_rc(rc, mysql);
 
-  rc= mysql_query(my, "CREATE user sha256user@localhost identified with sha256_password by 'foo'");
+  sprintf(query, "CREATE user 'sha256user'@'%s' identified with sha256_password by 'foo'", this_host);
+  rc= mysql_query(my, query);
   check_mysql_rc(rc, my);
+
   if (!mysql_real_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0))
   {
     diag("error: %s", mysql_error(mysql));
@@ -1108,7 +1126,8 @@ static int test_auth256(MYSQL *my)
     return FAIL;
   }
   mysql_close(mysql);
-  rc= mysql_query(my, "DROP USER sha256user@localhost");
+  sprintf(query, "DROP USER 'sha256user'@'%s'", this_host);
+  rc= mysql_query(my, query);
   check_mysql_rc(rc, mysql);
   return OK;
 }
@@ -1218,7 +1237,6 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
   }
-  diag("charset: %s", mysql_character_set_name(mysql));
   FAIL_IF(strcmp("latin2", mysql_character_set_name(mysql)), "Expected charset latin2");
   mysql_close(mysql);
 
@@ -1228,18 +1246,18 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
     return FAIL;
 
   fprintf(fp, "[client]\n");
-  fprintf(fp, "default-character-set=latin1\n");
-  fprintf(fp, "[client-server]\n");
-  fprintf(fp, "default-character-set=latin1\n");
-  fprintf(fp, "[client-mariadb]\n");
   fprintf(fp, "default-character-set=utf8\n");
-  fprintf(fp, "[connection]\n");
+  fprintf(fp, "[client-server]\n");
+  fprintf(fp, "default-character-set=utf8\n");
+  fprintf(fp, "[client-mariadb]\n");
   fprintf(fp, "default-character-set=latin2\n");
 
   fclose(fp);
 
   rc= mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, "./mdev13100.cnf");
   check_mysql_rc(rc, mysql);
+  rc= mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
+  check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
                              password, schema, port, socketname, 0))
@@ -1247,13 +1265,13 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
   }
-
-  FAIL_IF(strcmp("utf8", mysql_character_set_name(mysql)), "Expected charset utf8");
+  diag("character set: %s", mysql_character_set_name(mysql));
+  FAIL_IF(strcmp("latin2", mysql_character_set_name(mysql)), "Expected charset latin2");
   mysql_close(mysql);
 
   remove("./mdev13100.cnf");
 
-  return OK; 
+  return OK;
 }
 
 static int test_conc276(MYSQL *unused __attribute__((unused)))
@@ -1285,175 +1303,272 @@ static int test_conc276(MYSQL *unused __attribute__((unused)))
   return OK;
 }
 
-static int test_mdev9059_1(MYSQL *my __attribute__((unused)))
-{
-  MYSQL *mysql= mysql_init(NULL);
-  int rc;
-
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @a:=1");
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @b:=1");
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SELECT @a,@b");
-
-  FAIL_IF(!my_test_connect(mysql, hostname, username, password, schema,
-                              port, socketname,
-                              CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS), mysql_error(mysql));
-
-  rc= mysql_query(mysql, "SET @a:=0, @b:=0");
-  check_mysql_rc(rc, mysql);
-
-  mysql_close(mysql);
-  return OK;
-}
-
-static int test_mdev9059_2(MYSQL *my __attribute__((unused)))
-{
-  MYSQL *mysql= mysql_init(NULL);
-
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @a:=1");
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @b:=1");
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "this is an error, connect should fail");
-
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-
-  FAIL_IF(!mysql_errno(mysql), "Error expected");
-
-  mysql_close(mysql);
-  return OK;
-}
-
-static int test_mdev9059_3(MYSQL *my __attribute__((unused)))
-{
-  MYSQL *mysql= mysql_init(NULL);
-
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @a:=1");
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "SET @b:=1");
-
-  my_test_connect(mysql, hostname, "thisuserdoesntexist", password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-
-  diag("error (expected): %s", mysql_error(mysql));
-  FAIL_IF(!mysql_errno(mysql), "Error expected");
-
-  mysql_close(mysql);
-  return OK;
-}
-
-static int test_conc277(MYSQL *my __attribute__((unused)))
-{
-  MYSQL *mysql;
-
-
-  mysql_library_end();
-  mysql= mysql_init(NULL);
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-
-  FAIL_IF(mysql_errno(mysql), "No error expected");
-
-  mysql_close(mysql);
-  return OK;
-}
-
-/* CONC-297: libmysql incompatibility: According to the documentation
-   load data local infile will be enabled by passing NULL pointer 
-   or a pointer to an unsigned int value */
-static int test_conc297(MYSQL *my __attribute__((unused)))
-{
-  MYSQL *mysql;
-  int  local_infile= 0;
-
-  mysql= mysql_init(NULL);
-  mysql_optionsv(mysql, MYSQL_OPT_LOCAL_INFILE, NULL);
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-  FAIL_IF(mysql_errno(mysql), "No error expected");
-
-  mysql_get_option(mysql, MYSQL_OPT_LOCAL_INFILE, &local_infile);
-
-  FAIL_IF(!local_infile, "local infile was not enabled");
-  mysql_close(mysql);
-
-  mysql= mysql_init(NULL);
-  local_infile= 0;
-  mysql_optionsv(mysql, MYSQL_OPT_LOCAL_INFILE, &local_infile);
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-  FAIL_IF(mysql_errno(mysql), "No error expected");
-  mysql_get_option(mysql, MYSQL_OPT_LOCAL_INFILE, &local_infile);
-
-  FAIL_IF(local_infile, "local infile was not disabled");
-  mysql_close(mysql);
-
-  mysql= mysql_init(NULL);
-  local_infile= 1;
-  mysql_optionsv(mysql, MYSQL_OPT_LOCAL_INFILE, &local_infile);
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-  FAIL_IF(mysql_errno(mysql), "No error expected");
-  mysql_get_option(mysql, MYSQL_OPT_LOCAL_INFILE, &local_infile);
-
-  FAIL_IF(!local_infile, "local infile was not enabled");
-  mysql_close(mysql);
-
-  return OK;
-}
-
-static int test_mdev14647(MYSQL *my __attribute__((unused)))
+static int test_expired_pw(MYSQL *my)
 {
   MYSQL *mysql;
   int rc;
-  const char *data= "0";
-  size_t length= 0;
-  long server_capabilities;
+  char query[512];
+  unsigned char expire= 1;
 
-  mysql= mysql_init(NULL);
-  my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname,
-                  CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS);
-  FAIL_IF(mysql_errno(mysql), "No error expected");
-
-  if (!mariadb_connection(mysql))
+  if (mariadb_connection(my) ||
+     !(my->server_capabilities & CLIENT_CAN_HANDLE_EXPIRED_PASSWORDS))
   {
-    diag("MySQL doesn't send SESSION_TRACK_STATE_CHANGE");
+    diag("Server doesn't support password expiration");
     return SKIP;
   }
+  sprintf(query, "DROP USER 'foo'@'%s'", this_host);
+  rc= mysql_query(my, query);
 
-  mariadb_get_infov(mysql, MARIADB_CONNECTION_SERVER_CAPABILITIES, &server_capabilities);
+  sprintf(query, "CREATE USER 'foo'@'%s' IDENTIFIED BY 'foo'", this_host);
+  rc= mysql_query(my, query);
+  check_mysql_rc(rc, my);
 
-  if (!(server_capabilities & CLIENT_SESSION_TRACKING) ||
-      !mariadb_connection(mysql))
+  sprintf(query, "GRANT ALL ON *.* TO 'foo'@'%s'", this_host);
+  rc= mysql_query(my, query);
+  check_mysql_rc(rc, my);
+
+  sprintf(query, "ALTER USER 'foo'@'%s' PASSWORD EXPIRE", this_host);
+  rc= mysql_query(my, query);
+  check_mysql_rc(rc, my);
+
+  mysql= mysql_init(NULL);
+
+  my_test_connect(mysql, hostname, "foo", "foo", schema,
+                  port, socketname, 0);
+
+  FAIL_IF(!mysql_errno(mysql), "Error expected");
+  mysql_close(mysql);
+
+  mysql= mysql_init(NULL);
+  mysql_optionsv(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, &expire);
+
+  my_test_connect(mysql, hostname, "foo", "foo", schema,
+                  port, socketname, 0);
+
+  diag("error: %d %s", mysql_errno(mysql), mysql_error(mysql));
+  FAIL_IF(mysql_errno(mysql) != ER_MUST_CHANGE_PASSWORD &&
+          mysql_errno(mysql) != ER_MUST_CHANGE_PASSWORD_LOGIN, "Error 1820/1862 expected");
+
+  mysql_close(mysql);
+
+  sprintf(query, "DROP USER 'foo'@'%s'", this_host);
+  rc= mysql_query(my, query);
+  check_mysql_rc(rc, my);
+
+  return OK;
+}
+
+static int test_conc315(MYSQL *mysql)
+{
+  int rc;
+  const char *csname;
+
+  if (!is_mariadb)
+    return SKIP;
+
+  mysql_get_optionv(mysql, MYSQL_SET_CHARSET_NAME, (void *)&csname);
+  diag("csname=%s", csname);
+  FAIL_UNLESS(strcmp(csname, MARIADB_DEFAULT_CHARSET) == 0, "Wrong default character set");
+
+  rc= mysql_change_user(mysql, username, password, schema);
+  check_mysql_rc(rc, mysql);
+  mysql_get_optionv(mysql, MYSQL_SET_CHARSET_NAME, (void *)&csname);
+  FAIL_UNLESS(strcmp(csname, MARIADB_DEFAULT_CHARSET) == 0, "Wrong default character set");
+  return OK;
+}
+#ifndef WIN32
+static int test_conc317(MYSQL *unused __attribute__((unused)))
+{
+  MYSQL *mysql;
+  my_bool reconnect = 0;
+  FILE *fp= NULL;
+  const char *env= getenv("MYSQL_TMP_DIR");
+  char cnf_file1[FN_REFLEN + 1];
+
+  if (travis_test)
+    return SKIP;
+
+  if (!env)
+    env= "/tmp";
+
+  setenv("HOME", env, 1);
+
+  snprintf(cnf_file1, FN_REFLEN, "%s%c.my.cnf", env, FN_LIBCHAR);
+
+  FAIL_IF(!access(cnf_file1, R_OK), "access");
+
+  mysql= mysql_init(NULL);
+  fp= fopen(cnf_file1, "w");
+  FAIL_IF(!fp, "fopen");
+
+  fprintf(fp, "[client]\ndefault-character-set = latin2\nreconnect= 1\n");
+  fclose(fp);
+
+  mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
+  my_test_connect(mysql, hostname, username, password,
+                  schema, 0, socketname, 0);
+
+  remove(cnf_file1);
+
+  FAIL_IF(strcmp(mysql_character_set_name(mysql), "latin2"), "expected charset latin2");
+  mysql_get_optionv(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  FAIL_IF(reconnect != 1, "expected reconnect=1");
+  mysql_close(mysql);
+  return OK;
+}
+
+static int test_conc327(MYSQL *unused __attribute__((unused)))
+{
+  MYSQL *mysql;
+  my_bool reconnect = 0;
+  FILE *fp1= NULL, *fp2= NULL;
+  const char *env= getenv("MYSQL_TMP_DIR");
+  char cnf_file1[FN_REFLEN + 1];
+  char cnf_file2[FN_REFLEN + 1];
+
+  if (travis_test)
+    return SKIP;
+
+  if (!env)
+    env= "/tmp";
+
+  setenv("HOME", env, 1);
+
+  snprintf(cnf_file1, FN_REFLEN, "%s%c.my.cnf", env, FN_LIBCHAR);
+  snprintf(cnf_file2, FN_REFLEN, "%s%c.my.tmp", env, FN_LIBCHAR);
+
+  FAIL_IF(!access(cnf_file1, R_OK), "access");
+
+  fp1= fopen(cnf_file1, "w");
+  fp2= fopen(cnf_file2, "w");
+  FAIL_IF(!fp1 || !fp2, "fopen failed");
+
+  fprintf(fp1, "!include %s\n", cnf_file2);
+  
+  fprintf(fp2, "[client]\ndefault-character-set = latin2\nreconnect= 1\n");
+  fclose(fp1);
+  fclose(fp2);
+
+  mysql= mysql_init(NULL);
+  mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
+  my_test_connect(mysql, hostname, username, password,
+                  schema, 0, socketname, 0);
+
+  remove(cnf_file1);
+  remove(cnf_file2);
+
+  FAIL_IF(strcmp(mysql_character_set_name(mysql), "latin2"), "expected charset latin2");
+  mysql_get_optionv(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  FAIL_IF(reconnect != 1, "expected reconnect=1");
+  mysql_close(mysql);
+
+  snprintf(cnf_file1, FN_REFLEN, "%s%cmy.cnf", env, FN_LIBCHAR);
+  fp1= fopen(cnf_file1, "w");
+  fp2= fopen(cnf_file2, "w");
+  FAIL_IF(!fp1 || !fp2, "fopen failed");
+
+  fprintf(fp2, "!includedir %s\n", env);
+  
+  fprintf(fp1, "[client]\ndefault-character-set = latin2\nreconnect= 1\n");
+  fclose(fp1);
+  fclose(fp2);
+  mysql= mysql_init(NULL);
+  mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, cnf_file2);
+  my_test_connect(mysql, hostname, username, password,
+                  schema, 0, socketname, 0);
+
+  remove(cnf_file1);
+  remove(cnf_file2);
+
+  FAIL_IF(strcmp(mysql_character_set_name(mysql), "latin2"), "expected charset latin2");
+  mysql_get_optionv(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  FAIL_IF(reconnect != 1, "expected reconnect=1");
+  mysql_close(mysql);
+
+  return OK;
+}
+#endif
+
+static int test_conc332(MYSQL *unused __attribute__((unused)))
+{
+  int rc;
+  MYSQL *mysql= mysql_init(NULL);
+  int server_status1, server_status2;
+
+  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8mb4");
+
+  my_test_connect(mysql, hostname, username, password, schema,
+                  port, socketname, 0);
+
+  FAIL_IF(mysql_errno(mysql), "Error during connect");
+
+  mariadb_get_infov(mysql, MARIADB_CONNECTION_SERVER_STATUS, &server_status1);
+  diag("server_status: %d", server_status1);
+
+  if (server_status1 & SERVER_STATUS_AUTOCOMMIT)
+    rc= mysql_query(mysql, "SET autocommit= 0");
+  else
+    rc= mysql_query(mysql, "SET autocommit= 1");
+  check_mysql_rc(rc, mysql);
+  mariadb_get_infov(mysql, MARIADB_CONNECTION_SERVER_STATUS, &server_status2);
+  diag("server_status after changing autocommit: %d", server_status2);
+
+  rc= mysql_change_user(mysql, username, password, schema);
+  check_mysql_rc(rc, mysql);
+
+  mariadb_get_infov(mysql, MARIADB_CONNECTION_SERVER_STATUS, &server_status2);
+  diag("server_status after mysql_change_user: %d", server_status2);
+  if (server_status1 != server_status2)
   {
-    diag("Server doesn't support session tracking or doesn't send SESSION_TRACK_STATE_CHANGE information");
+    diag("Expected server_status %d instead of %d", server_status1, server_status2);
     mysql_close(mysql);
+    return FAIL;
+  }
+  mysql_close(mysql);
+  return OK;
+}
+
+static int test_conc351(MYSQL *unused __attribute__((unused)))
+{
+  int rc;
+  const char *data;
+  size_t len;
+  MYSQL *mysql= mysql_init(NULL);
+  ulong capabilities= 0;
+
+  my_test_connect(mysql, hostname, username, password, schema,
+                  port, socketname, 0);
+
+  FAIL_IF(mysql_errno(mysql), "Error during connect");
+
+  mariadb_get_infov(mysql, MARIADB_CONNECTION_SERVER_CAPABILITIES, &capabilities);
+  if (!(capabilities & CLIENT_SESSION_TRACKING))
+  {
+    mysql_close(mysql);
+    diag("Server doesn't support session tracking (cap=%lu)", mysql->server_capabilities);
     return SKIP;
   }
 
-  rc= mysql_query(mysql, "SET SESSION session_track_state_change = 1");
+  FAIL_IF(mysql_session_track_get_first(mysql, SESSION_TRACK_SCHEMA, &data, &len), "expected session track schema");
+
+  rc= mysql_query(mysql, "SET @a:=1");
   check_mysql_rc(rc, mysql);
 
-  rc= mysql_session_track_get_first(mysql, SESSION_TRACK_STATE_CHANGE, (const char **)&data, &length);
-
-  FAIL_IF(length != 1, "expected length=1");
-  FAIL_IF(data[0] != 0x31, "expected 1");
+  FAIL_IF(!mysql_session_track_get_first(mysql, SESSION_TRACK_SCHEMA, &data, &len), "expected no schema tracking information");
 
   mysql_close(mysql);
   return OK;
 }
 
 struct my_tests_st my_tests[] = {
-  {"test_mdev14647", test_mdev14647, TEST_CONNECTION_NONE, 0, NULL,  NULL},
-  {"test_conc297", test_conc297, TEST_CONNECTION_NONE, 0, NULL,  NULL},
-  {"test_conc277", test_conc277, TEST_CONNECTION_NONE, 0, NULL,  NULL},
-  {"test_mdev9059_1", test_mdev9059_1, TEST_CONNECTION_NONE, 0, NULL,  NULL},
-  {"test_mdev9059_2", test_mdev9059_2, TEST_CONNECTION_NONE, 0, NULL,  NULL},
-  {"test_mdev9059_3", test_mdev9059_3, TEST_CONNECTION_NONE, 0, NULL,  NULL},
+  {"test_conc351", test_conc351, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conc332", test_conc332, TEST_CONNECTION_NONE, 0, NULL, NULL},
+#ifndef WIN32
+  {"test_conc327", test_conc327, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+  {"test_conc317", test_conc317, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+#endif
+  {"test_conc315", test_conc315, TEST_CONNECTION_NEW, 0, NULL,  NULL},
+  {"test_expired_pw", test_expired_pw, TEST_CONNECTION_DEFAULT, 0, NULL,  NULL},
   {"test_conc276", test_conc276, TEST_CONNECTION_NONE, 0, NULL,  NULL},
   {"test_mdev13100", test_mdev13100, TEST_CONNECTION_DEFAULT, 0, NULL,  NULL},
   {"test_auth256", test_auth256, TEST_CONNECTION_DEFAULT, 0, NULL,  NULL},
